<?php

namespace Teknisa\CNC\Services;

use Teknisa\Libs\Util\Environment;

use Teknisa\CNC\Utilities\UtilCNC;

use Teknisa\CNC\Factories\Cnc00400ConciliacaoManualFactory;

class Cnc00400ConciliacaoManualService {
    protected $entityManager;
    protected $environment;
    protected $factory;
    protected $util;
    protected $errosPadroes = array(
        '001' => 'Total de crédito difere do total de débito.',
        '002' => 'Data fora do período contábil.',
        '003' => 'Partida %s - Conta contábil não cadastrada.',
        '004' => 'Partida %s - Histórico padrão não cadastrado.',
        '005' => 'Partida %s - Código comercial não cadastrado.',
        '006' => 'Partida %s - Valor da partida zerado.',
        '007' => 'Partida %s - Valor de rateio zerado.',
        '008' => 'Partida %s - Filial de rateio não cadastrada.',
        '009' => 'Partida %s - Centro de custo não cadastrado para filial informada.',
        '010' => 'Partida %s - Soma dos rateios difere do valor da partida.',
        '011' => 'Partida %s - Conta contábil sem código comercial informado.',
        '012' => 'Partida %s - Conta Contábil Gerencial sem rateio informado.',
        '013' => 'Partida %s - Centro de custo deve ser informado.',
        '014' => 'Produto %s sem conta contábil.',
        '015' => 'Lançamento fora do período contábil para o financeiro. (BCO, PAG e REC)',
        '016' => 'Lançamento fora do período contábil de documentos fiscais de entrada. (DFE)',
        '017' => 'Lançamento fora do período contábil para documentos fiscais de saída. (DFS)'
    );

    
    public function __construct (\Doctrine\ORM\EntityManager $entityManager,Environment $environment, Cnc00400ConciliacaoManualFactory $factory, UtilCNC $util) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->factory = $factory;
        $this->util = $util;
    }
    
    public function selectConciliacoes ($filter) {
        if ($this->util->compareDates($filter['DTVENDA'][0], $filter['DTVENDA'][1]))
            throw new \Exception("MOUNTH_INTERVAL", 1);
        if(isset($filter["ISEQUAL"])){
            if ($filter["ISEQUAL"] == false)
            throw new \Exception("IS_EQUAL_FILIAL", 2);
        }
        $datasets[] = $this->leftGridSelect($filter);
        $datasets[] = $this->rightGridSelect($filter);
        return $datasets;
    }
    
    public function leftGridSelect ($filter) {
        return $this->factory->leftGridSelect(array(
            ":NRORG"        => $filter["NRORG"],
            ":NRESTABELEC"  => $filter["NRESTABELEC"],
            ":TNRESTABELEC" => $filter["TNRESTABELEC"],
            ":NRADQUIRENTE" => $filter["NRADQUIRENTE"],
            ":TNRADQUIRENTE"=> $filter["TNRADQUIRENTE"],
            ":NRPRODUTOADQ" => $filter["NRPRODUTOADQ"],
            ":TNRPRODUTOADQ"=> $filter["TNRPRODUTOADQ"],
            ":DTVENDA_MIN"  => $filter["DTVENDA"][0],
            ":DTVENDA_MAX"  => $filter["DTVENDA"][1],
            ":CDNSU"        => $filter["CDNSU"],
            ":TCDNSU"       => $filter["TCDNSU"],
            ":STATUS"       => $filter["STATUS"],
            ":TSTATUS"      => $filter["TSTATUS"]
        ));
    }
    
    public function rightGridSelect ($filter) {
        
        $params = array(
                ":NRORG"        => $filter["NRORG"],
                ":NRESTABELEC"  => $filter["NRESTABELEC"],
                ":TNRESTABELEC" => $filter["TNRESTABELEC"],
                ":NRPRODUTOADQ" => $filter["NRPRODUTOADQ"],
                ":TNRPRODUTOADQ"=> $filter["TNRPRODUTOADQ"],
                ":DTVENDA_MIN"  => $filter["DTVENDA"][0],
                ":DTVENDA_MAX"  => $filter["DTVENDA"][1],
                ":CDNSU"        => $filter["CDNSU"],
                ":TCDNSU"       => $filter["TCDNSU"],
                ":STATUS"       => $filter["STATUS"],
                ":TSTATUS"      => $filter["TSTATUS"]);
        
        if($filter["IDVISPRODUTOADQ"] == 'S'){
            return $this->factory->rightGridSelectVisibleProducts($params);        
        } else{
            return $this->factory->rightGridSelect($params);
        }
    
    }
    
    public function selectConciliacoesPagto ($filter) {
        $result = $this->factory->selectConciliacoesPagto(array(
            ":NRORG"        => $filter["NRORG"]["value"],
            ":NRESTABELEC"  => $filter["NRESTABELEC"]["value"],
            ":NRADQUIRENTE" => $filter["NRADQUIRENTE"]["value"],
            ":DTVENDA_MIN"  => $filter["DTVENDA"]["value"][0],
            ":DTVENDA_MAX"  => $filter["DTVENDA"]["value"][1],
            ":CDNSU"        => $filter["NSU"]["value"],
            ":TCDNSU"       => $filter["TNSU"]["value"],
            ":VALINICIAL"   => $filter["VALINICIAL"]["value"],
            ":UVALINICIAL"  => $filter["UVALINICIAL"]["value"],
            ":VALFINAL"     => $filter["VALFINAL"]["value"],
            ":UVALFINAL"    => $filter["UVALFINAL"]["value"],
            ":TIPOPGTO"     => $filter["TIPOPGTO"]["value"],
            ":STATUS"       => $filter["STATUS"]["value"] === "P" ? "I" : $filter["STATUS"]["value"],
            ":TSTATUS"      => $filter["TSTATUS"]["value"]
        ));
        return $result;
    }
    
    public function selectTabSisMovimentacoes ($filter) {
        $result = $this->factory->selectTabSisMovimentacoes(array(
            ":NRORG"        => $filter["NRORG"]["value"],
            ":NRCONCLANC"   => $filter["NRCONCLANC"]["value"],
            ":ISNULL"       => $filter["ISNULL"]["value"],
            ":DTVENDA_MIN"  => $filter["DTVENDA"]["value"][0],
            ":DTVENDA_MAX"  => $filter["DTVENDA"]["value"][1],
            ":IDCONCILIADO" => $filter["IDCONCILIADO"]["value"]
        ));
        
        return $result;
    }
    
    public function selectTabAdqMovimentacoes ($filter) {
        $result = $this->factory->selectTabAdqMovimentacoes(array(
            ":NRORG"        => $filter["NRORG"]["value"],
            ":NRCONCLANC"   => $filter["NRCONCLANC"]["value"],
            ":ISNULL"       => $filter["ISNULL"]["value"],
            ":DTVENDA_MIN"  => $filter["DTVENDA"]["value"][0],
            ":DTVENDA_MAX"  => $filter["DTVENDA"]["value"][1],
            ":IDCONCILIADO" => $filter["IDCONCILIADO"]["value"]
        ));
    
        return $result;
    }
    
    public function selectTabVndMovimentacoes ($filter) {
        $result = $this->factory->selectTabVndMovimentacoes(array(
            ":NRORG"        => $filter["NRORG"]["value"],
            ":NRCONCLANC"   => $filter["NRCONCLANC"]["value"],
            ":NRADQUIRENTE" => $filter["NRADQUIRENTE"]["value"],
            ":ISNULL"       => $filter["ISNULL"]["value"],
            ":DTVENDA_MIN"  => $filter["DTVENDA"]["value"][0],
            ":DTVENDA_MAX"  => $filter["DTVENDA"]["value"][1],
            ":IDCONCILIADO" => $filter["IDCONCILIADO"]["value"]
        ));
        
        return $result;
    }
    
    public function selectTabRetMovimentacoes ($filter) {
        $result = $this->factory->selectTabRetMovimentacoes(array(
            ":NRORG"            => $filter["NRORG"]["value"],
            ":NRCONCLANCPGTO"   => $filter["NRCONCLANCPGTO"]["value"],
            ":NRADQUIRENTE"     => $filter["NRADQUIRENTE"]["value"],
            ":ISNULL"           => $filter["ISNULL"]["value"],
            ":DTVENDA_MIN"      => $filter["DTVENDA"]["value"][0],
            ":DTVENDA_MAX"      => $filter["DTVENDA"]["value"][1],
            ":IDCONCILIADO"     => $filter["IDCONCILIADO"]["value"]
        ));
        
        return $result;
    }
    
    public function desconciliacaoManualAdq ($params, $tabsis) {
        $bool = true;
        
        $bool = $this->factory->nullifyRowsAtConsMovCaixa(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANC"]
        ));
        
        $bool = $bool && $this->factory->nullifyRowsAtRetVendaAdq(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANC"]
        ));
        
        $isPgtoConc = false;
        $concPagamentos = $params["NRCONCLANCPGTO"];
        if (!is_null($concPagamentos)) {
            $concPagamentos = is_array($concPagamentos) ? $concPagamentos : [$concPagamentos];
            $isPgtoConc = true;
        }
        
        if ($isPgtoConc) {
            foreach($concPagamentos as $concPgto) {
                $bool = $bool && $this->factory->nullifyRowsAtRetFinancAdq(array(
                    ":NRORG"        => $params["NRORG"],
                    ":NRCONCLANC"   => $concPgto
                ));
            }
        }
        
        // deleta a divconciliacao com o conclanc
        foreach ($tabsis as $row) {
            $bool = $bool && $this->factory->deleteFromDivConciliacaoConvmovcaixa(array(
                ":NRORG"        => $params["NRORG"],
                ":NRCONCLANC"   => $row["NRCONCLANC"],
                ":NRCONSMOVCAIXA"   => $row["NRCONSMOVCAIXA"]
            ));
        }
        
        // deleta a divconciliacao com o conclanc para a retfinancadq
        if ($isPgtoConc) {
            foreach ($concPagamentos as $concPgto) {
                $bool = $bool && $this->factory->deleteFromDivConciliacao(array(
                    ":NRORG"        => $params["NRORG"],
                    ":NRCONCLANC"   => $concPgto
                ));
            }
        }
        
        //deleta a conciliacaolanc
        $bool = $bool && $this->factory->deleteFromConciliacaoLanc(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANC"]
        ));
        
        //deleta a conciliacaolanc
        if ($isPgtoConc) {
            foreach ($concPagamentos as $concPgto) {
                $bool = $bool && $this->factory->deleteFromConciliacaoLanc(array(
                    ":NRORG"        => $params["NRORG"],
                    ":NRCONCLANC"   => $concPgto
                ));
            }
        }
        
        return $bool;
    }
    
    public function desconciliacaoManualSis ($params) {
        $bool = true;
        $contaReceber = $this->factory->verifyContaReceberInfo(array(
            ":NRLANCTONF" => $params["NRLANCTONF"],
            ":CDFILIAL"   => $params["CDFILIAL"]
        ));

        if($contaReceber && !empty($contaReceber)){
            $boletoReceber = $this->factory->verifyBoletReceberInfo(array(
                ":CDFILIBOLREC" => $contaReceber["CDFILIBOLREC"],
                ":NRLANBOLREC"  => $contaReceber["NRLANBOLREC"]
            ));
            
            $VRFINALBREC = $boletoReceber['VRFINALBREC'] - $contaReceber['VRATUAREC'];
            
            if ($VRFINALBREC > 0) {
                $bool = $this->factory->updateBoletReceber(array(
                    ":CDFILIAL"     => $contaReceber["CDFILIBOLREC"],
                    ":NRLANBOLREC"  => $contaReceber["NRLANBOLREC"],
                    ":VRFINALBREC"  => $VRFINALBREC,
                    ":VRBOLREC"     => $VRFINALBREC
                ));
                
                $bool = $bool && $this->factory->nullifyBoletContaReceber(array(
                    ":NRLANCTONF" => $params["NRLANCTONF"],
                    ":CDFILIAL"   => $params["CDFILIAL"]
                ));
                
                $infoConsmovcaixa = $this->verifyInfoConsmov($params);
                
                $P_NRLANBOLREC = $this->util->newCode("BOLETRECEBER".$params["CDFILIAL"], "10");
    
                $infoTpReceb = $this->factory->verifyInfoTpReceb(array(
                    ":CDTIPORECE"       => $params["CDTIPORECE"],
                    ":CDFILIAL"         => $params["CDFILIAL"]
                ));
                $this->factory->createBoletRec(array(
                    "P_NRLANBOLREC"     => $P_NRLANBOLREC,
                    "P_CDFILIAL"        => $params["CDFILIAL"],
                    "P_CDCLIENTE"       => $params["CDCLIENTE"],
                    "P_CDEMPRESA"       => $params["CDEMPRESA"],
                    "P_VRBOLREC"        => $contaReceber['VRATUAREC'],
                    "P_DTVENBOL"        => $params["DTCREDITO"],
                    "P_CDTIPORECE"      => $params["CDTIPORECE"],
                    "P_NRCHEQUEVEND"    => null,
                    "P_CDBANCHEQVEN"    => null,
                    "P_CDAGECHEQVEN"    => null,
                    "P_CDCNTCHEQVEN"    => null,
                    "P_CDREDUCONT"      => $infoTpReceb["CDREDUCONT"],
                    "P_CDTPBAIXA"       => null,
                    "P_DTVENDAECF"      => null
                ));
    
                $this->factory->updateContaReceberBol(array(
                    ":CDFILIBOLREC" => $contaReceber["CDFILIAL"],
                    ":NRLANBOLREC"  => $P_NRLANBOLREC,
                    ":NRLANCTONF"   => $contaReceber["NRLANCTONF"],
                    ":CDFILIAL"     => $contaReceber["CDFILIAL"],
                    ":NRPARCELAREC" => $contaReceber["NRPARCELAREC"] 
                ));
                
                $totOldBolet = $this->factory->recalculaBoleto(array(
                    ":CDFILIBOLREC" => $boletoReceber["CDFILIBOLREC"],
                    ":NRLANBOLREC"  => $boletoReceber["NRLANBOLREC"]
                ));
                
                $this->factory->updateBoletReceber(array(
                    ":VRFINALBREC"  =>  $totOldBolet["TOTALCONTAS"],
                    ":VRBOLREC"     =>  $totOldBolet["TOTALCONTAS"],
                    ":CDFILIAL"     =>  $totOldBolet["CDFILIBOLREC"],
                    ":NRLANBOLREC"  =>  $totOldBolet["NRLANBOLREC"]
                ));
    
                $this->factory->updateTprecboleto(array(
                    ":NRLANBOLREC"  =>  $totOldBolet["NRLANBOLREC"],
                    ":VRTPRECBOLR"  =>  $totOldBolet["TOTALCONTAS"],
                    ":CDFILIAL"     =>  $totOldBolet["CDFILIBOLREC"]
                ));
            }
            
            return $bool;
        }else{
            throw new \Exception("Não foi possível desconciliar movimentações. Pois não há informações de conta a receber!",1);
        }
    }
    
    public function atualizaSistema ($params) {
        $bool = true;
        
        $movCaixaInfo = $this->factory->getMovCaixa(array(
            ":CDFILIAL"   => $params["CDFILIAL"],
            ":CDCAIXA"    => $params["CDCAIXA"],
            ":NRSEQUMOVI" => $params["NRSEQUMOVI"],
            ":DTABERCAIX" => $params["DTABERCAIX"]
        ));
        
        $diasVenc = $this->factory->calcNrDiasVenc(array(
            ":CDFILIAL"   => $params["CDFILIAL"],
            ":CDTIPORECE" => $movCaixaInfo["CDTIPORECE"]
        ))['V_NRDIASVENC'];
        
        if($diasVenc==null){
            $diasVenc = 0;
        }
                
        $dataVenc = $this->factory->verifyDataVenc(array(
            ":CDFILIAL"    => $params["CDFILIAL"],
            ":DTVENDA"     => $params["DTVENDA"],
            ":NRDIASVENC"  => $diasVenc,
            ":CDTIPORECE"  => $movCaixaInfo["CDTIPORECE"]
        ));
        
        $bool = $this->factory->updateConsMovCaixaDesconciliacao(array(
            ":NRCONSMOVCAIXA" => $params["NRCONSMOVCAIXA"],
            ":CDFILIAL"       => $params["CDFILIAL"],
            ":CDCAIXA"        => $params["CDCAIXA"],
            ":NRSEQUMOVI"     => $params["NRSEQUMOVI"],
            ":DTABERCAIX"     => $params["DTABERCAIX"],
            ":CDNSU"          => $movCaixaInfo['CDNSUHOSTTEF'],
            ":VRVENDA"        => $movCaixaInfo['VRMOVIVEND'],
            ":CDTIPORECE"     => $movCaixaInfo['CDTIPORECE'],
            ":DTVENDA"        => $movCaixaInfo['DTMOVIMCAIXA'],
            ":DTVENCIMENTO"   => $dataVenc['V_DTVENCIMENTO']
        ));
        
        return $bool;
    }
    
    public function desconciliacaoManualDePagamento ($params) {
        $bool = $this->factory->nullifyConcPgtoAtRetVendaAdq(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANCPGTO"   => $params["NRCONCLANCPGTO"][0]
        ));
        
        $bool = $bool && $this->factory->nullifyRowsAtRetFinancAdq(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANCPGTO"][0]
        ));
        
        $bool = $bool && $this->factory->deleteFromDivConciliacao(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANCPGTO"][0]
        ));
        
        $bool = $bool && $this->factory->deleteFromConciliacaoLanc(array(
            ":NRORG"        => $params["NRORG"],
            ":NRCONCLANC"   => $params["NRCONCLANCPGTO"][0]
        ));
        
        return $bool;
    }
    
    public function insertIntoConciliacao ($params, $nrconclanc, $date) {
        return $this->factory->insertAtConciliacaoLanc(array(
            ":NRCONCLANC"       => $nrconclanc,
            ":DTCONCILIACAO"    => $date,
            ":IDTPCONCILIACAO"  => "M",
            ":NRDIVCONCILIACAO" => null,
            ":NRORG"            => $params[0]["NRORG"],
            ":IDATIVO"          => "S",
            ":DTINCLUSAO"       => $date,
            ":DTULTATU"         => $date,
            ":NRORGINCLUSAO"    => $params[0]["NRORG"],
            ":CDOPERINCLUSAO"   => $this->environment->getCdOperador(),
            ":NRORGULTATU"      => $params[0]["NRORG"],
            ":CDOPERULTATU"     => $this->environment->getCdOperador()
        ));
    }
                                          // $params Left (Adquirente)       //Right Movimentacao do sistema
    public function conciliacaoManualAdq ($params, $nrconclanc, $isOneToOne, $rightRows) {
        $bool = true;
        $IDSITUACAO = $params[0]["isSolved"] == false ? "C" : "O";
        foreach ($params as $row) {
            $keys = is_array($row["CHAVE"]) ? $row["CHAVE"] : [$row["CHAVE"]];
            foreach ($keys as $key) {
                $bool = $bool && $this->factory->updateRetVendaAdq(array(
                    ":NRCONCLANC"       => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => $IDSITUACAO,
                    ":NRRETVENDAADQ"    => $key
                ));
            }
            
            if($isOneToOne){
                $NRLANCTONF = $rightRows[0]["NRLANCTONF"];
                $CDFILIALNF = $rightRows[0]["CDFILIALNF"];
                $key = $row["CHAVE"];
                $bool = $bool && $this->factory->updateRetVendaAdqLanctoNf(array(
                    ":NRLANCTONF"       =>  $NRLANCTONF,
                    ":CDFILIALNF"       =>  $CDFILIALNF,
                    ":NRRETVENDAADQ"    =>  $key
                ));
            }else {
                // associamos a consmovcaixa com a retvendaadq, em casos de conciliacao NxN utilizando o valor bruto
                foreach ($rightRows as $consmov) {
                    if ($consmov['VRBRUTVENDA'] == $row['VRBRUTVENDA']) {
                        $key = $row["CHAVE"];
                        $bool = $bool && $this->factory->updateRetVendaAdqLanctoNf(array(
                            ":NRLANCTONF"       =>  $consmov['NRLANCTONF'],
                            ":CDFILIALNF"       =>  $consmov['CDFILIALNF'],
                            ":NRRETVENDAADQ"    =>  $key
                        ));
                    }
                }
            }
        }
        
        return $bool;
    }
    
    public function processingBeforeSystemConciliation ($params) {
        $bool = true;
        foreach ($params as $row) {
            $keys = is_array($row["CHAVE"]) ? $row["CHAVE"] : [$row["CHAVE"]];
            foreach ($keys as $key) {
                $bool = $bool && $this->factory->apagaContasPrevistas(array(
                    ":CDFILIAL"   => $row["CDFILIAL"],
                    ":NRLANCTONF" => $row["NRLANCTONF"]
                ));
                
                $bool = $bool && $this->factory->updateConsMovCaixaBeforeDelete(array(
                    ":CDFILIALNF" => $row["CDFILIALNF"],
                    ":NRLANCTONF" => $row["NRLANCTONF"]
                ));
                
                $bool = $bool && $this->factory->deleteFromNotaFiscal(array(
                    ":CDFILIAL"   => $row["CDFILIAL"],
                    ":NRLANCTONF" => $row["NRLANCTONF"]
                ));
            }
        }
        
        return $bool;
    }
                                          //Params movimentacao do sistema                  // left movimentacoes da adquirente
    public function conciliacaoManualSis ($params, $nrconclanc, $isOneToOne, $nradquirente, $leftRows) {
        $bool = true;
        foreach ($params as $row) {
            $keys = is_array($row["CHAVE"]) ? $row["CHAVE"] : [$row["CHAVE"]];
            foreach ($keys as $key) {
                $bool = $bool && $this->factory->updateConsMovCaixa(array(
                    ":NRCONCLANC"       => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => $isOneToOne ? "O" : "C",
                    ":CHAVE"            => $key
                ));
                $this->factory->verifyInfoConsmov(array(
                    ":NRCONSMOVCAIXA"   => $row["CHAVE"],
                    ":NRORG"            => $row["NRORG"]
                ));
                $this->factory->updateContRecebDesc(array(
                    ":CDFILIALNF"       => $row["CDFILIALNF"],
                    ":NRLANCTONF"       => $row["NRLANCTONF"],
                    ":NRPARCELAREC"     => $row["NRPARCELAREC"]
                ));
            }
        }
        return $bool;
    }
    
    public function updateContaReceberNF($params, $nradquirente, $leftRows){
        $bool = true;
        $IDGERAFINANC = $this->factory->verificaGeraFinanc($nradquirente)["IDGERAFINANC"];
        
            $connection = $this->entityManager->getConnection();
        
        if($IDGERAFINANC == "S"){
            
            // como em outros momentos pega as informacoes da consmovcaixa e caso nao tenha nota e conta ele roda a rotina de gerar financeiro FINANCEIRO_FOS_CNC.
            $infoConsmovcaixa = $this->verifyInfoConsmov($params[0]);
            
            if(empty($infoConsmovcaixa)){
                throw new \Exception("ERR/"."Parametrização de integração incompleta. Favor verificar as parametrizações do financeiro reconciliar pela tela de Reconciliação Automática.", 1);
            }
            
            //Pega o cliente da adquirente, que pode ser parametrizado na tela de Parametrizacao do Produto
            $cdcliente = $this->factory->getCdcliente($nradquirente)["CDCLIENTE"];
            $params['paramsToSearchCnc'] = $params[0];
            $params['paramsToSearchCnc']['NRESTABELEC'] = $this->factory->findNrestabelec(array(':CDESTABELEC' => $leftRows[0]['CDESTABELEC']))['NRESTABELEC'];
            $params['paramsToSearchCnc']['NRADQUIRENTE'] = $nradquirente['NRADQUIRENTE'];
            $params['paramsToSearchCnc']['NRPRODUTOADQ'] = $this->factory->findNrproduto(array(':NMPRODUTOADQ' => $leftRows[0]['NMPRODUTOADQ']))['NRPRODUTOADQ'];
            $params['paramsToSearchCnc']['P_GRAVAERRO'] = 'N';
            $params['paramsToSearchCnc']['P_DTINICIAL'] = NULL;
            $params['paramsToSearchCnc']['P_DTFINAL']   = NULL;
            $params['paramsToSearchCnc']['TIPOPARAM']   = 'TIPOCONTA';
            
            // Utilizando a variavel acima busca parametros de Tipo de conta
            $tpconta = $this->searchParamsCnc($params['paramsToSearchCnc']);
            if(!$tpconta["CDTPCTRECEB"]){
                throw new \Exception("ERR/"."O produto da venda selecionada não possui tipo de conta relacionada. Favor verificar sua parametrização na tela Parametrização de Produtos para uma correta geração do financeiro.", 1);
            }
            
            // Realiza a verificacao se o tipo de conta e igual ao parametrizado e a data de vencimento, caso nao seja, update esses valores na conta a receber.
            if ($infoConsmovcaixa["CDTPCTRECEB"] != $tpconta["CDTPCTRECEB"] || $infoConsmovcaixa["DTATUAVENREC"] != $infoConsmovcaixa["DTVENCIMENTO"]) {
                
                $this->factory->updateContaReceber(array(
                    ":CDTPCTRECEB"  => $tpconta["CDTPCTRECEB"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIALNF"],
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":DTATUAVENREC" => $infoConsmovcaixa["DTVENCIMENTO"],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"],
                    ":IDREPFOSCNC"  => "N"
                ));
            } else {
                $this->factory->updateContaReceberGeral(array(
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIALNF"],
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"],
                    ":IDREPFOSCNC"  => "N"
                ));
            }
            
            // Busca informacoes da contareceber
            $contaRecebeInfo = $this->factory->verifyContaReceberInfo(array(
                ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"]
            ));
            
            // Busca informacoes do boleto associado
            $boletoRecebeInfo = $this->factory->verifyBoletReceberInfo(array(
                ":CDFILIBOLREC" => $contaRecebeInfo["CDFILIBOLREC"],
                ":NRLANBOLREC"  => $contaRecebeInfo["NRLANBOLREC"]
            ));
            
            // reduz o valor da conta no boleto 
            $VRFINALBREC = $boletoRecebeInfo['VRFINALBREC'] - $contaRecebeInfo['VRATUAREC'];
            
            // Refaz taxa de servico e ajustes de conta boleto e tipo de recebimento do boleto
            $this->factory->refazTaxaServFinanc(array(
                ":NRCONSMOVCAIXA"   => $infoConsmovcaixa["NRCONSMOVCAIXA"]
            ));
            
            // ????? nao sei pq faz isso 
            $this->factory->clearBolRecebFromContaReceb(array(
                ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                ":NRPARCELAREC"   => $infoConsmovcaixa["NRPARCELAREC"]
            ));
            
            if($VRFINALBREC != 0){
                $this->factory->updateBoletReceber(array(
                    ":VRFINALBREC"  =>  $VRFINALBREC,
                    ":VRBOLREC"     =>  $VRFINALBREC,
                    ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                    ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                ));
                
                    
                //tipo de recebimento do boleto
                $this->factory->updateTprecboleto(array(
                    ":VRTPRECBOLR"  =>  $VRFINALBREC,
                    ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                    ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                ));
            } else {
                
                    // ????? nao sei pq faz isso   Repete o processo 
                $this->factory->upateBoletoConta(array(
                    ":P_CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                    ":P_NRLANCTONF"  => $infoConsmovcaixa["NRLANCTONF"]
                ));
                
                //Deleta boleto cujo valor é zero para não acumular lixo de memória no banco de dados.
                $this->factory->deleteBoletReceber(array(
                    ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                    ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                ));
                
                //parte nova entender se deve ou nao fazer
                //Deleta tipo de recebimento do boleto cujo valor é zero para não acumular lixo de memória no banco de dados.
                $this->factory->deleteTiporeceBolet(array(
                    ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                    ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                ));
            }
            
            $nrdiasvenc = $this->factory->calcNrDiasVenc(array(
                ":CDTIPORECE"   =>  $infoConsmovcaixa["CDTIPORECE"],
                ":CDFILIAL"     =>  $infoConsmovcaixa["CDFILIAL"]
            ))["V_NRDIASVENC"];

            if($nrdiasvenc==null){
                $nrdiasvenc = 0;
            }
            
            $DTVENBOLREC = $leftRows[0]["DTCREDITO"];
            
            // Verificar se ja houve baixa no boleto aqui 
            
            // Ponto que iremos colocar em script para corrigir contas sem boleto.
            $boletoAReceber = $this->factory->existBoletRec(array(
                ":CDTPCTRECEB"  => $tpconta["CDTPCTRECEB"],
                ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                ":CDCLIENTE"    => $infoConsmovcaixa["CDCLIENTE"],
                ":CDEMPRESA"    => $infoConsmovcaixa["CDEMPRESA_ESTABELEC"],
                ":DTVENBOLREC"  => $DTVENBOLREC
            ));
            
            
            if($boletoAReceber){
                $this->factory->updateContaReceberBol(array(
                    ":CDFILIBOLREC" => $boletoAReceber["CDFILIAL"],
                    ":NRLANBOLREC"  => $boletoAReceber["NRLANBOLREC"],
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"]
                ));
                
                $totalContas = $this->factory->recalculaTotalBoleto(array(
                    ":CDFILIBOLREC" => $boletoAReceber["CDFILIAL"],
                    ":NRLANBOLREC"  => $boletoAReceber["NRLANBOLREC"]
                ));
                $VRFINALBREC = $boletoAReceber["VRFINALBREC"] + $contaRecebeInfo["VRATUAREC"];
                
                // Adicionado um recalculo do valor do boleto baseado nas contas associadas
                $VRTOTALCONTAS = $totalContas['TOTALCONTAS'];
                
                //updatado os dois valores, o inicial e o original
                $boletoRecebeParams = array(
                    ":VRFINALBREC"  =>  $VRTOTALCONTAS,
                    ":VRBOLREC"     =>  $VRTOTALCONTAS,
                    ":CDFILIAL"     =>  $boletoAReceber['CDFILIAL'],
                    ":NRLANBOLREC"  =>  $boletoAReceber['NRLANBOLREC'],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"]
                );

                // Alterado
                $this->factory->updateBoletReceber(array(
                    ":VRFINALBREC"  =>  $boletoRecebeParams[':VRFINALBREC'],
                    ":VRBOLREC"     =>  $boletoRecebeParams[':VRBOLREC'],
                    ":CDFILIAL"     =>  $boletoRecebeParams[':CDFILIAL'],
                    ":NRLANBOLREC"  =>  $boletoRecebeParams[':NRLANBOLREC']
                ));
                
                $this->factory->updateTprecboleto(array(
                    ":NRLANBOLREC"  =>  $boletoAReceber['NRLANBOLREC'],
                    ":VRTPRECBOLR"  =>  $VRFINALBREC,
                    ":CDFILIAL"     =>  $boletoAReceber['CDFILIAL']
                ));
            
            } else {
                $P_NRLANBOLREC = $this->util->newCode("BOLETRECEBER".$infoConsmovcaixa["CDFILIAL"], "10");
                
                $infoTpReceb = $this->factory->verifyInfoTpReceb(array(
                    ":CDTIPORECE"       => $infoConsmovcaixa["CDTIPORECE"],
                    ":CDFILIAL"         => $infoConsmovcaixa["CDFILIAL"]
                ));
                
                $this->factory->createBoletRec(array(
                    "P_NRLANBOLREC"     => $P_NRLANBOLREC,
                    "P_CDFILIAL"        => $infoConsmovcaixa["CDFILIAL"],
                    "P_CDCLIENTE"       => $infoConsmovcaixa["CDCLIENTE"],
                    "P_CDEMPRESA"       => $infoConsmovcaixa["CDEMPRESA_ESTABELEC"],
                    "P_VRBOLREC"        => $contaRecebeInfo['VRATUAREC'],
                    "P_DTVENBOL"        => $DTVENBOLREC,
                    "P_CDTIPORECE"      => $infoConsmovcaixa["CDTIPORECE"],
                    "P_NRCHEQUEVEND"    => null,
                    "P_CDBANCHEQVEN"    => null,
                    "P_CDAGECHEQVEN"    => null,
                    "P_CDCNTCHEQVEN"    => null,
                    "P_CDREDUCONT"      => $infoTpReceb["CDREDUCONT"],
                    "P_CDTPBAIXA"       => null,
                    "P_DTVENDAECF"      => null
                ));
                
                $this->factory->updateContaReceberBol(array(
                    ":CDFILIBOLREC" => $infoConsmovcaixa["CDFILIAL"],
                    ":NRLANBOLREC"  => $P_NRLANBOLREC,
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"]
                ));
                
            }
            
            $params['paramsToSearchCnc']['TIPOPARAM']   = 'TIPOOPER';
            $tipooper = $this->searchParamsCnc($params['paramsToSearchCnc']);
            if(!$tipooper["CDTIPOOPER"]){
                throw new \Exception("ERR/"."O produto da venda selecionada não possui tipo de operação relacionada. Favor verificar sua parametrização na tela Parametrização de Produtos para uma correta geração do financeiro.", 1);
            }

            if($tipooper["CDTIPOOPER"] != $infoConsmovcaixa["CDTIPOOPER"] 
            || $infoConsmovcaixa["CDEMPRESA_ESTABELEC"] != $infoConsmovcaixa["CDEMPRESA_NF"] 
            || $infoConsmovcaixa["CDINSCESTA_ESTABELEC"] !=  $infoConsmovcaixa["CDINSCESTA_NF"] 
            || $infoConsmovcaixa["CDINSCMUNI_ESTABELEC"] != $infoConsmovcaixa["CDINSCMUNI_NF"] 
            || $infoConsmovcaixa["CDSERIE_ESTABELEC"] != $infoConsmovcaixa["CDSERIE_NF"]){
                $this->factory->updateNotaFiscal(array(
                    ":CDEMPRESA"    => $infoConsmovcaixa["CDEMPRESA_ESTABELEC"],
                    ":CDINSCESTA"   => $infoConsmovcaixa["CDINSCESTA_ESTABELEC"],
                    ":CDINSCMUNI"   => $infoConsmovcaixa["CDINSCMUNI_ESTABELEC"],
                    ":CDSERIE"      => $infoConsmovcaixa["CDSERIE_ESTABELEC"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIALNF"],
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":CDTIPOOPER"   => $tipooper["CDTIPOOPER"],
                    ":NRNOTAFISC"   => $leftRows[0]["CDNSU"],
                    ":CDCLIENTE"    => $infoConsmovcaixa["CDCLIENTE"]
                ));
            }
            $this->factory->updateClienteNf(array(
                ":CDCLIENTE"    => $cdcliente,
                ":CDFILIAL"     => $infoConsmovcaixa["CDFILIALNF"],
                ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"]
            ));
        }
        
        return $bool;
    }
                                         // Sistema    //Adquirente
    public function unificaBoletoPorProduto($rightRows, $leftRows){
        $bool = true;
        $nradquirente['NRADQUIRENTE'] = $leftRows[0]["NRADQUIRENTE"];
        $nmprodutoadq = $leftRows[0]["NMPRODUTOADQ"];
        $nmestabelec = $leftRows[0]["NMESTABELEC"];
        $cdestabelec = $leftRows[0]['CDESTABELEC'];
        
        foreach ($leftRows as $row) {
            if($row["NRADQUIRENTE"] != $nradquirente['NRADQUIRENTE'] || $row["NMPRODUTOADQ"] != $nmprodutoadq){
                return $bool;
            }
        }
        
        foreach ($leftRows as $row) {
            if($row["CDESTABELEC"] != $cdestabelec){
                return $bool;
            }
        }

        foreach ($rightRows as $row) {
            if($row["NMPRODUTOADQ"] != $nmprodutoadq){
                return $bool;
            }
        }

        $IDGERAFINANC = $this->factory->verificaGeraFinanc($nradquirente)["IDGERAFINANC"];
        if($IDGERAFINANC == "S"){
            foreach ($rightRows as $row) {
                $infoConsmovcaixa = $this->verifyInfoConsmov($row);
                $cdcliente = $this->factory->getCdcliente($nradquirente)["CDCLIENTE"];
                $rightRows['paramsToSearchCnc'] = $row;
                $rightRows['paramsToSearchCnc']['NRESTABELEC'] = $this->factory->findNrestabelec(array(
                                                                                ':CDESTABELEC'  => $cdestabelec))['NRESTABELEC'];
                                                                                
                $rightRows['paramsToSearchCnc']['NRADQUIRENTE'] = $nradquirente['NRADQUIRENTE'];
                $rightRows['paramsToSearchCnc']['NRPRODUTOADQ'] = $this->factory->findNrproduto(array(
                                                                                ':NMPRODUTOADQ' => $nmprodutoadq))['NRPRODUTOADQ'];
                                                                                
                $rightRows['paramsToSearchCnc']['P_GRAVAERRO'] = 'N';
                $rightRows['paramsToSearchCnc']['P_DTINICIAL'] = NULL;
                $rightRows['paramsToSearchCnc']['P_DTFINAL']   = NULL;
                $rightRows['paramsToSearchCnc']['TIPOPARAM']   = 'TIPOCONTA';
                
                $tpconta = $this->searchParamsCnc($rightRows['paramsToSearchCnc']);
                if(!$tpconta["CDTPCTRECEB"]){
                    throw new \Exception("ERR/"."O produto da venda selecionada não possui tipo de conta relacionada. Favor verificar sua parametrização na tela Parametrização de Produtos para uma correta geração do financeiro.", 1);
                }
          
                $this->factory->updateContaReceberGeral(array(
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIALNF"],
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"],
                    ":IDREPFOSCNC"  => "N"
                ));
            
                $contaRecebeInfo = $this->factory->verifyContaReceberInfo(array(
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"]
                ));
                
                $boletoRecebeInfo = $this->factory->verifyBoletReceberInfo(array(
                    ":CDFILIBOLREC" => $contaRecebeInfo["CDFILIBOLREC"],
                    ":NRLANBOLREC"  => $contaRecebeInfo["NRLANBOLREC"]
                ));
                
                $VRFINALBREC = $boletoRecebeInfo['VRFINALBREC'] - $contaRecebeInfo['VRATUAREC'];
                
                $this->factory->clearBolRecebFromContaReceb(array(
                    ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                    ":NRPARCELAREC"   => $infoConsmovcaixa["NRPARCELAREC"]
                ));
                    
                if($VRFINALBREC != 0){
                    $this->factory->updateBoletReceber(array(
                        ":VRFINALBREC"  =>  $VRFINALBREC,
                        ":VRBOLREC"  =>  $VRFINALBREC,
                        ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                        ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                    ));
                    
                    //tipo de recebimento do boleto
                    $this->factory->updateTprecboleto(array(
                        ":VRTPRECBOLR"  =>  $VRFINALBREC,
                        ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                        ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                    ));
                    
                } else {
                    $this->factory->upateBoletoConta(array(
                        ":P_CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                        ":P_NRLANCTONF"  => $infoConsmovcaixa["NRLANCTONF"]
                    ));

                    //Deleta boleto cujo valor é zero para não acumular lixo de memória no banco de dados.
                    $this->factory->deleteBoletReceber(array(
                        ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                        ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                    ));
                
                    //parte nova entender se deve ou nao fazer
                    //Deleta tipo de recebimento do boleto cujo valor é zero para não acumular lixo de memória no banco de dados.
                    $this->factory->deleteTiporeceBolet(array(
                        ":CDFILIAL"     =>  $contaRecebeInfo['CDFILIBOLREC'],
                        ":NRLANBOLREC"  =>  $contaRecebeInfo['NRLANBOLREC']
                    ));
                }
                
                $nrdiasvenc = $this->factory->calcNrDiasVenc(array(
                    ":CDTIPORECE"   =>  $infoConsmovcaixa["CDTIPORECE"],
                    ":CDFILIAL"     =>  $infoConsmovcaixa["CDFILIAL"]
                ))["V_NRDIASVENC"];
                    
                if($nrdiasvenc==null){
                    $nrdiasvenc = 0;
                }
                
                $DTVENBOLREC = $this->factory->verifyDataVenc(array(
                    ":CDFILIAL"     =>  $infoConsmovcaixa["CDFILIAL"],
                    ":DTVENDA"      =>  $infoConsmovcaixa["DTVENDA"],
                    ":NRDIASVENC"   =>  $nrdiasvenc,
                    ":CDTIPORECE"   =>  $infoConsmovcaixa["CDTIPORECE"]
                ))["V_DTVENCIMENTO"];
                    
                $boletoAReceber = $this->factory->existBoletRec(array(
                    ":CDTPCTRECEB"  => $tpconta["CDTPCTRECEB"],
                    ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                    ":CDCLIENTE"    => $infoConsmovcaixa["CDCLIENTE"],
                    ":CDEMPRESA"    => $infoConsmovcaixa["CDEMPRESA_ESTABELEC"],
                    ":DTVENBOLREC"  => $DTVENBOLREC
                ));
                if($boletoAReceber){
                    $this->factory->updateContaReceberBol(array(
                        ":CDFILIBOLREC" => $boletoAReceber["CDFILIAL"],
                        ":NRLANBOLREC"  => $boletoAReceber["NRLANBOLREC"],
                        ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                        ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                        ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"]
                    ));
                    $VRFINALBREC = $boletoAReceber["VRFINALBREC"] + $contaRecebeInfo["VRATUAREC"];
                    
                    $boletoRecebeParams = array(
                        ":VRFINALBREC"  =>  $VRFINALBREC,
                        ":CDFILIAL"     =>  $boletoAReceber['CDFILIAL'],
                        ":NRLANBOLREC"  =>  $boletoAReceber['NRLANBOLREC']
                    );
    
                    // Alterado
                    $this->factory->updateBoletReceber(array(
                        ":VRFINALBREC"  =>  $VRFINALBREC,
                        ":VRBOLREC"  =>  $VRFINALBREC,
                        ":CDFILIAL"     =>  $boletoRecebeParams[':CDFILIAL'],
                        ":NRLANBOLREC"  =>  $boletoRecebeParams[':NRLANBOLREC']
                    ));
                    
                    $this->factory->updateTprecboleto(array(
                        ":NRLANBOLREC"  =>  $boletoAReceber['NRLANBOLREC'],
                        ":VRTPRECBOLR"  =>  $VRFINALBREC,
                        ":CDFILIAL"     =>  $boletoAReceber['CDFILIAL']
                    ));
                
                }else{
                    $P_NRLANBOLREC = $this->util->newCode("BOLETRECEBER".$infoConsmovcaixa["CDFILIAL"], "10");
                    
                    $infoTpReceb = $this->factory->verifyInfoTpReceb(array(
                        ":CDTIPORECE"       => $infoConsmovcaixa["CDTIPORECE"],
                        ":CDFILIAL"         => $infoConsmovcaixa["CDFILIAL"]
                    ));
                    $this->factory->createBoletRec(array(
                        "P_NRLANBOLREC"     => $P_NRLANBOLREC,
                        "P_CDFILIAL"        => $infoConsmovcaixa["CDFILIAL"],
                        "P_CDCLIENTE"       => $infoConsmovcaixa["CDCLIENTE"],
                        "P_CDEMPRESA"       => $infoConsmovcaixa["CDEMPRESA_ESTABELEC"],
                        "P_VRBOLREC"        => $contaRecebeInfo['VRATUAREC'],
                        "P_DTVENBOL"        => $DTVENBOLREC,
                        "P_CDTIPORECE"      => $infoConsmovcaixa["CDTIPORECE"],
                        "P_NRCHEQUEVEND"    => null,
                        "P_CDBANCHEQVEN"    => null,
                        "P_CDAGECHEQVEN"    => null,
                        "P_CDCNTCHEQVEN"    => null,
                        "P_CDREDUCONT"      => $infoTpReceb["CDREDUCONT"],
                        "P_CDTPBAIXA"       => null,
                        "P_DTVENDAECF"      => null
                    ));

                    $this->factory->updateContaReceberBol(array(
                        ":CDFILIBOLREC" => $infoConsmovcaixa["CDFILIAL"],
                        ":NRLANBOLREC"  => $P_NRLANBOLREC,
                        ":NRLANCTONF"   => $infoConsmovcaixa["NRLANCTONF"],
                        ":CDFILIAL"     => $infoConsmovcaixa["CDFILIAL"],
                        ":NRPARCELAREC" => $infoConsmovcaixa["NRPARCELAREC"] 
                    ));
                }
            }
         
        }

        return $bool;
    }
    
    public function verificaGeraFinanc($nradquirente){
        return $this->factory->verificaGeraFinanc($nradquirente);
    }
    
    public function recontabilizaVendaConciliada($nrconclanc, $nradquirente){
        $bool = true;
        $IDGERAFINANC = $this->factory->verificaGeraFinanc($nradquirente)["IDGERAFINANC"];
        if($IDGERAFINANC == "S"){
            $nrconclanc = array(
                ":NRCONCLANC" => $nrconclanc
            );
            $paramContaCncContabiliza = $this->factory->findVendasConciliadas($nrconclanc);
            if(!empty($paramContaCncContabiliza) && $paramContaCncContabiliza !== '' && $paramContaCncContabiliza !== null) {
                $paramsContalibizaOld = array(
                    ":NRCONCLANC"   => $nrconclanc[":NRCONCLANC"],
                    ":NRPRODUTOADQ" => $paramContaCncContabiliza["NRPRODUTOADQ"]  
                );
                $paramOldContaCncContabiliza = $this->factory->findVendasConciliadasOld($paramsContalibizaOld);
                if(!empty($paramOldContaCncContabiliza) && $paramOldContaCncContabiliza !== '' && $paramOldContaCncContabiliza !== null) {
                    if($paramOldContaCncContabiliza["VRESPERADO"] != $paramOldContaCncContabiliza["VRENCONTRADO"]){
                        $this->factory->contabilizaContaCnc($paramOldContaCncContabiliza);
                    }
                }
                $this->factory->contabilizaContaCnc($paramContaCncContabiliza);
            }
        }
        
        return $bool;
        
    }
    
    public function getErrors($params) {
        $CDEMPRESA	   = $params[0]["value"];
		$NRLANCCTBTEMP = $params[1]["value"];
		$DSERROS       = [];
		$params = array(
			':CDEMPRESA' 		 => $CDEMPRESA,
			':NRLANCCTBTEMP'      => $NRLANCCTBTEMP
		);
		$error = $this->factory->getError($params);
		foreach ($error as $key => $value) {
		    $NRSEQCTBTEMP = $error[$key]["NRSEQCTBTEMP"];
		    $DSERRO = $this->errosPadroes[$error[$key]["CDERROTEMP"]];
		    if(strrpos($DSERRO, "%s") !== false){
		        $DSERRO = str_replace('%s', $NRSEQCTBTEMP, $DSERRO);
                $DSERROS[$key]["DSERRO"] = $DSERRO;
            } else {
                $DSERROS[$key]["DSERRO"] = $DSERRO;
            }
		}
		return $DSERROS;
	}
    
    public function updateConsWithVnd ($params) {
        $leftRow  = $params[0][0];
        $rightRow = $params[1][0];
        
        $vndData = $this->factory->selectRetVendaInfo(array(
            ":NRORG"         => $leftRow["NRORG"],
            ":NRRETVENDAADQ" => $leftRow["CHAVE"]
        ));

        $bool = false;
        $hasTipoRece = is_null($vndData["CDTIPORECE"]);
        
        
        if (!$hasTipoRece) {
            $bool = $this->factory->updateConsWithVndValues(array(
                ":NRCONSMOVCAIXA" => $rightRow["CHAVE"],
                ":NRORG"          => $vndData["NRORG"],
                ":CDNSU"          => $vndData["CDNSU"],
                ":CDTIPORECE"     => $vndData["CDTIPORECE"],
                ":NRAUTORIZACAO"  => $vndData["NRAUTORIZACAO"],
                ":NRQTPARCELA"    => $vndData["NRQTPARCELA"],
                ":NRPARCELA"      => $vndData["NRPARCELA"],
                ":DTVENDA"        => $vndData["DTVENDA"],
                ":DTVENCIMENTO"   => $vndData["DTCREDITO"],
                ":VRVENDA"        => $vndData["VRBRUTVENDA"]
            ));
            
        } else{
            $nameProdutoAdq = $this->factory->getNameProdutoadq(array(
                ":NRPRODUTOADQ" => $vndData["NRPRODUTOADQ"]
            ));
            throw new \Exception("ERR/"."O Produto ".$nameProdutoAdq["NMPRODUTOADQ"]." não possui tipo de recebimento relacionado.", 1);
        }
        return $bool;
    }
    
    public function conciliacaoManualVnd ($params) {
        $nrconclanc = intval($this->util->newCode("CONCILIACAOLANC", "20"));
        $date = date("d/m/Y");
        
        $bool = $this->factory->insertAtConciliacaoLanc(array(
            ":NRCONCLANC"       => $nrconclanc,
            ":DTCONCILIACAO"    => $date,
            ":IDTPCONCILIACAO"  => "M",
            ":NRDIVCONCILIACAO" => null,
            ":NRORG"            => $params[0]["NRORG"],
            ":IDATIVO"          => "S",
            ":DTINCLUSAO"       => $date,
            ":DTULTATU"         => $date,
            ":NRORGINCLUSAO"    => $params[0]["NRORG"],
            ":CDOPERINCLUSAO"   => $this->environment->getCdOperador(),
            ":NRORGULTATU"      => $params[0]["NRORG"],
            ":CDOPERULTATU"     => $this->environment->getCdOperador()
        ));
        
        foreach ($params as $row) {
            foreach ($row["NRRETVENDAADQ"] as $value) {
                $bool = $bool && $this->factory->updatePgtoRetVendaAdq(array(
                    ":NRCONCLANCPGTO"   => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => "C",
                    ":NRRETVENDAADQ"    => $value
                ));
            }
            foreach ($row["NRRETFINANCADQ"] as $value) {
                $bool = $bool && $this->factory->updateRetFinancAdq(array(
                    ":NRCONCLANC"   => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => "C",
                    ":NRRETFINANCADQ"   => $value
                ));
            }
        }
        
        return $bool;
    }
    
    public function conciliacaoManualRet ($params) {
        $nrconclanc = intval($this->util->newCode("CONCILIACAOLANC", "20"));
        $date = date("d/m/Y");
        
        $bool = $this->factory->insertAtConciliacaoLanc(array(
            ":NRCONCLANC"       => $nrconclanc,
            ":DTCONCILIACAO"    => $date,
            ":IDTPCONCILIACAO"  => "M",
            ":NRDIVCONCILIACAO" => null,
            ":NRORG"            => $params[0]["NRORG"],
            ":IDATIVO"          => "S",
            ":DTINCLUSAO"       => $date,
            ":DTULTATU"         => $date,
            ":NRORGINCLUSAO"    => $params[0]["NRORG"],
            ":CDOPERINCLUSAO"   => $this->environment->getCdOperador(),
            ":NRORGULTATU"      => $params[0]["NRORG"],
            ":CDOPERULTATU"     => $this->environment->getCdOperador()
        ));
        
        foreach ($params as $row) {
            foreach ($row["NRRETVENDAADQ"] as $value) {
                $bool = $bool && $this->factory->updatePgtoRetVendaAdq(array(
                    ":NRCONCLANCPGTO"   => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => "C",
                    ":NRRETVENDAADQ"    => $value
                ));
            }
            foreach ($row["NRRETFINANCADQ"] as $value) {
                $bool = $bool && $this->factory->updateRetFinancAdq(array(
                    ":NRCONCLANC"   => $nrconclanc,
                    ":IDCONCILIADO"     => "C",
                    ":IDSITUACAO"       => "C",
                    ":NRRETFINANCADQ"   => $value
                ));
            }
        }
        
        return $bool;
    }
    
    public function checkItemCorrecao ($DSITEMCORRECAO) {
        return $this->factory->checkItemCorrecao(array(
            ":DSITEMCORRECAO" => $DSITEMCORRECAO
        ));
    }
    
    public function deleteItemCorVenda ($NRCONSMOVCAIXA, $NRORG) {
        return $this->factory->deleteItemCorVenda(array(
            ":NRCONSMOVCAIXA" => $NRCONSMOVCAIXA,
            ":NRORG" => $NRORG
        ));
    }
    
    public function insertItemCorVenda ($params) {
        $params["NRITEMCORVENDA"] = intval($this->util->newCode("ITEMCORVENDA", "10"));
        $date = date("d/m/Y");
        $this->factory->insertItemCorVenda(array(
            ":NRITEMCORVENDA" => $params["NRITEMCORVENDA"],
            ":NRCONSMOVCAIXA" => $params["NRCONSMOVCAIXA"],
            ":NRITEMCORRECAO" => $params["NRITEMCORRECAO"],
            ":VRANTIGO"       => $params["VRANTIGO"],
            ":VRNOVO"         => $params["VRNOVO"],
            ":NRORG"          => $params["NRORG"]
        ));
    }
    
	public function deleteErrors ($params) {
        $CDEMPRESA   = $params[0]['CDEMPRESA'];
		$NRLANCCTBTEMP = $params[0]['NRLANCCTBTEMP'];

		$paramethers = array(
			':CDEMPRESA'   => $CDEMPRESA,
			':NRLANCCTBTEMP' => $NRLANCCTBTEMP
		);

		$this->factory->deleteErrors($paramethers);
	}
	
	public function searchParamsCnc($params){
	   $params = array(
	       ':P_NRESTABELEC'   =>      $params['NRESTABELEC'],
	       ':P_NRADQUIRENTE'  =>      $params['NRADQUIRENTE'],
	       ':P_NRPRODUTOADQ'  =>      $params['NRPRODUTOADQ'],
	       ':P_TIPOPARAM'     =>      $params['TIPOPARAM'],
	       ':P_GRAVAERRO'     =>      $params['P_GRAVAERRO'],
	       ':P_DTINICIAL'     =>      $params['P_DTINICIAL'],
	       ':P_DTFINAL'       =>      $params['P_DTFINAL']
	   );
	   return $this->factory->searchParamsCnc($params);
	}
	
	public function verifyInfoConsmov($params){
	    $param = array(
	        ':NRCONSMOVCAIXA'   =>      $params['CHAVE'],
	        ':NRORG'            =>      $params['NRORG']
	    );
	    return $this->factory->verifyInfoConsmov($param);
	}
	
    public function checkItemDivergencia ($DSTIPODIV) {
        return $this->factory->checkItemDivergencia(array(
            ":DSTIPODIV" => $DSTIPODIV
        ));
    }
    
    public function insertDivergenciaConciliacao ($params, $nrconclanc) {
        $params["NRDIVCONCILIACAO"] = intval($this->util->newCode("DIVCONCILIACAO", "10"));
        $date = date("d/m/Y");
        if($params["NRTPDIVERGENCIA"]==4){
            $params["VRNOVO"] = $this->factory->findNrproduto(array(
                ":NMPRODUTOADQ"  =>  $params["VRNOVO"]
            ))["NRPRODUTOADQ"];
            $params["VRANTIGO"] = $this->factory->findNrproduto(array(
                ":NMPRODUTOADQ"  =>  $params["VRANTIGO"]
            ))["NRPRODUTOADQ"];
        }

        
        $this->factory->insertDivergenciaConciliacao(array(
            ":NRDIVCONCILIACAO"      => $params["NRDIVCONCILIACAO"],
            ":NRCONCLANC"            => $nrconclanc,
            ":NRTPDIVERGENCIA"       => $params["NRTPDIVERGENCIA"],
            ":NRRETVENDAADQ"         => $params["NRRETVENDAADQ"],
            ":NRCONSMOVCAIXA"        => $params["NRCONSMOVCAIXA"],
            ":NRORG"                 => $params["NRORG"],
            ":VRESPERADO"            => $params["VRANTIGO"],
            ":VRENCONTRADO"          => $params["VRNOVO"]
        ));
    }

    public function verificaDataPeriodoContabil($params){
        return $this->factory->verificaDataPeriodoContabil(array(
            ":CDEMPRESA"            => $params["CDEMPRESA"],
            ":DTVENDA"              => $params["DTVENDA"]
        ));
    }
     
    public function vendaDentroCtrlFinanc($params, $filial){
        $dateCtrlFiancFilial = $this->factory->verificaDataControleFinancFilial(array(":CDFILIAL" => $filial));
        
        if($dateCtrlFiancFilial == 0) {
            $dateCtrlFianc = $this->factory->verificaDataControleFinanc(array());
            if($dateCtrlFianc == 0) {
                throw new \Exception("A Data de Início da Integração não está parametrizada. Realize este cadastro na tela 'Data de Início de Integração'.", 1);
            } else {
                $dateCtrlFianc = date('Y-m-d', strtotime(str_replace('/', '-', $dateCtrlFianc)));
            }  
        } else {
            $dateCtrlFianc = date('Y-m-d', strtotime(str_replace('/', '-', $dateCtrlFiancFilial)));
        }
        $dataVenda = date('Y-m-d', strtotime(str_replace('/', '-', $params["DTVENDA"])));
        
        //strtotime converte date de origem US para timestamp unix, permitindo comparação entre datas
        if(strtotime($dateCtrlFianc) <= strtotime($dataVenda)){
            return true;
        }else{
            return false;
        }
        
    }
    
    public function verificaEmpresaDataVenda($params){
        return $this->factory->verificaEmpresaDataVenda(array(
            ":NRCONSMOVCAIXA"       => $params["CHAVE"],
            ":NRORG"                => $params["NRORG"]
        ));
    }
    
    public function deleteDivConciliacao ($NRCONSMOVCAIXA, $NRORG) {
        return $this->factory->deleteDivConciliacao(array(
            ":NRCONSMOVCAIXA" => $NRCONSMOVCAIXA,
            ":NRORG" => $NRORG
        ));
    }
    
    public function verifyContReceb($params){
        return $this->factory->verifyContReceb(array(
            ":CDFILIAL"     => $params["CDFILIAL"],
            ":NRLANCTONF"   => $params["NRLANCTONF"]
        ));
    }
    

}