<?php

namespace Teknisa\CNC\Services;

use Teknisa\CNC\Utilities;
use Teknisa\CNC\Factories;

class Cnc01100SimuacaoAntecipada {

	protected $entityManager;
	protected $util;
	protected $simulaAntFactory;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager, Utilities\UtilCNC $util, Factories\Cnc01100SimulacaoAntecipadaFactory $simulaAntFactory)  {
		$this->entityManager     = $entityManager;
		$this->util              = $util;
		$this->simulaAntFactory  = $simulaAntFactory;
	}
	
	// Retorna true se a primeira data for mais recente ou igual à segunda.
	private function compareTwoDates ($olderDate, $youngerDate) {
        $olderDateTimeObj = \DateTime::createFromFormat('d/m/Y', $olderDate);
        $youngerDateTimeObj = \DateTime::createFromFormat('d/m/Y', $youngerDate);

        return $youngerDateTimeObj <= $olderDateTimeObj;
	}

	public function getAnticipatedSimulation($conditions) {
        $olderDate = (new \DateTime('now'))->format('d/m/Y');
        $youngerDate = $conditions["DTCREDITOINI"];
        
        // A previsão do pagamento deve ser posterior ao dia atual.
        if ($this->compareTwoDates($olderDate, $youngerDate)) {
            throw new \Exception("A data de Previsão de Pagamento da(s) venda(s) deve ser posterior ao dia atual.", 2); 
        }
        
        return $this->simulaAntFactory->getAnticipatedSimulation($conditions);
	}

    public function getAnticipatedParametrizedSimulation($params){
        $P_NRESTABELEC = $params[0]['value'];
        $P_NRADQ    = $params[1]['value'];
        $P_PRODUTO  = $params[2]['value'];

        $params = array(
            'P_NRESTABELEC'   => $P_NRESTABELEC,
            'P_NRADQ'      => $P_NRADQ,
            'P_PRODUTO'    => $P_PRODUTO
        );

        $standarts = $this->simulaAntFactory->callAnticipatedSimulationProcedure($params);
        return $standarts;
    }

}